#! /usr/bin/perl

# Auteur : Benoît MARCHAL - Université de Lorraine
#	Benoit.Marchal@univ-lorraine.fr

# Remarque : utilise des primitives du logiciel sympa (sympa.org) et aussi des portions de codes de celui-ci.

# Date : 20181031 (version 1.0)
#	Première version du script


=encoding utf-8

=head1 Name

copieListe = copie d'une liste en utilisant les procédures internes fournies par Sympa

=head1 Synopsis

copieListe [--source=<listeSource> --destination=<listeDestination> [--notest]] | [--help]

=head1 Description

Cette procédure va prendre une liste <listeSource> définie avec son domaine pour recopier sa définition (en incluant les propriétaires et modérateurs, mais par les abonnés) dans une liste <listeDestination>
Ces deux paramètres sont obligatoires.

=over 8

=item B<--source>

Prend une liste <nom>@<domaine>

=item B<--destination>

Prend une liste <nom>@<domaine>

=item B<--notest>

Effectue réellement l'action (sinon celle-ci est simulée)

=item B<--help>

Cette aide

=back

=cut

#use strict;

use English;
use Getopt::Long;

# Accès aux primitives de SYMPA
use lib '/usr/share/sympa/lib/';
use Conf;
use Sympa::List; 
use Sympa::Log; 

# Définition du compte qui est autorisé à effectuer l'action
# en liaison avec le scenario /etc/sympa/scenari/create_list.listmaster
# ligne : is_equal([sender], 'MONUTILSATEURQUEJEVEUXAUTORISE')        md5,smime       -> do_it
my $listmaster='MONUTILSATEURQUEJEVEUXAUTORISE';


my ($debug,$notest, $source, $destination, $help);
GetOptions(
    'debug!'		=> \$debug,
    'notest!'           => \$notest, 
    'source:s'          => \$source, 
    'destination:s'     => \$destination,
    'help!'		=> \$help
);

if ( defined($help)) {
    usage();
    exit(0);
}

# Chargement de la configuration de Sympa
my $sympa_conf_file = Sympa::Constants::CONFIG;
## Load sympa config
unless (Conf::load()) {
    printf STDERR
        "Impossible de charger le fichier de configuration %s ou bien un fichier robot.conf contient des erreurs. Je sors.\n",
        Conf::get_sympa_conf();
    exit 1;
}


# Changement de l'UID et GID pour passer avec l'utilsateur de Sympa (récupération du code de sympa.pl)
# Set the User ID & Group ID for the process
$GID = $EGID = (getgrnam(Sympa::Constants::GROUP))[2];
$UID = $EUID = (getpwnam(Sympa::Constants::USER))[2];

## Required on FreeBSD to change ALL IDs
## (effective UID + real UID + saved UID)
POSIX::setuid((getpwnam(Sympa::Constants::USER))[2]);
POSIX::setgid((getgrnam(Sympa::Constants::GROUP))[2]);

## Check if the UID has correctly been set (useful on OS X)
unless (($GID == (getgrnam(Sympa::Constants::GROUP))[2])
    && ($UID == (getpwnam(Sympa::Constants::USER))[2])) {
    die
        "Failed to change process user ID and group ID. Note that on some OS Perl scripts can't change their real UID. In such circumstances Sympa should be run via sudo.\n";
}

# Sets the UMASK
umask(oct($Conf::Conf{'umask'}));

# Ouverture des logs
my $log = Sympa::Log->instance;
$log->{level} = $Conf::Conf{'log_level'};
$log->openlog($Conf::Conf{'log_facility'} || $Conf::Conf{'syslog'},
    $Conf::Conf{'log_socket_type'});

$log->syslog(
    'notice',
    '%s : %s => %s',
    $PROGRAM_NAME,
    $source,
    $destination
    );

# Analyse des paramètres
if (!defined($source) or $source eq "" or !defined($destination) or $destination eq "" ) {
    usage();
    exit(1);
}

my @addr = ( $source, $destination );

foreach my $ad ( @addr ) {
    if ( !verifMail ($ad )) {
	printf  STDERR "%s n'est pas une adresse électronique valide dans Sympa\n", $ad;
	usage();
	exit(1);
    }
}

# On va charger la liste source
my $list = new Sympa::List ($source) ;
if( !defined($list)) {
	printf STDERR "Impossible de charger %s \n", $source;
}

my ($newList, $newRobot) = split /@/, $destination;
my $spindle = Sympa::Spindle::ProcessRequest->new(
    context      => $newRobot,
    action       => 'move_list',
    listname     => $newList,
    current_list => $list,
    mode         => 'copy',
    sender       => ($listmaster, (md5_check => 1)),
    
    scenario_context => {
	sender      => $listmaster,
    },
    );

if (defined($notest)) {
    if ($spindle and $spindle->spin) {
	$log->syslog(
	    'notice',
	    "Analyse du retour",
	    );
	foreach my $report (@{$spindle->{stash} || []}) {
	    if ($report->[1] eq 'notice') {
		$log->syslog(
		    'notice',
		    "Copie OK",
		    );
	    } else {
		$log->syslog(
		    'notice',
		    "Erreur pendant la copie",
		    );
		printf STDERR "Erreur pendant la copie : copie non effectué\n";
	    }
	}
    }
} else {
    printf "SIMULATION : copie de la liste %s vers la liste %s\n", $source, $destination;
}

exit 0;

# Fin du programme principal



sub usage {
    system("/usr/bin/perldoc $PROGRAM_NAME");
}

# Procédure simple de vérification du format d'une adresse de messagerie
# On profite des primitives de Sympa pour vérifier que la partie domaine appartient bien à un robot connu
sub verifMail {
    my $param = shift;
    my ($droite,$domain) = split /@/, $param;
    return 0 if ( $droite eq "" or $domain eq "");
    return 0 if ( !Conf::valid_robot($domain));
    return 1;
}
